#include "cstring.h"

namespace optimized_string
{

CString::CString()
:	m_szPtr(NULL),
	m_BufferLength(0)
{
	this->_Allocate(1);
	this->m_szPtr[0] = 0;
}

CString::CString(const CString& Other)
:	m_szPtr(NULL),
	m_BufferLength(0)
{
	this->Set(Other);
}

CString::CString(const char* szString, const size_t& Length)
:	m_szPtr(NULL),
	m_BufferLength(0)
{
	this->Set(szString, Length);
}

CString::~CString()
{
	if (this->m_szPtr)
	{
		delete[] this->m_szPtr;
	}
}

void CString::Reset()
{
	if (this->m_szPtr)
	{
		delete[] this->m_szPtr;
		this->m_szPtr = NULL;
	}
}

void CString::Set(const CString& Other)
{
	size_t Len = Other.Len();
	this->Reset();
	this->_Allocate(Len + 1);

	memcpy(this->m_szPtr, Other.m_szPtr, sizeof(char) * (Len + 1));
}

void CString::Set(const char* szString, const size_t& Len)
{
	this->Reset();
	this->_Allocate(Len + 1);

	memcpy(this->m_szPtr, szString, sizeof(char) * (Len + 1));
}

void CString::Append(const CString& Other, const size_t& Length)
{
	size_t CurLength = this->Len();
	this->_CheckGrow(Length, CurLength);

	// append
	memcpy(&this->m_szPtr[CurLength], Other.m_szPtr, sizeof(char) * Length);
	this->m_szPtr[CurLength + Length] = 0;
}

void CString::Append(const char* szString, const size_t& Length)
{
	size_t CurLength = this->Len();
	this->_CheckGrow(Length, CurLength);

	// append
	memcpy(&this->m_szPtr[CurLength], szString, sizeof(char) * Length);
	this->m_szPtr[CurLength + Length] = 0;
}

const bool CString::Compare(const CString& Other) const
{
	return (strcmp(this->m_szPtr, Other.m_szPtr) == 0);
}

const bool CString::Compare(const char* szString, const size_t& Length) const
{
	if (memcmp(this->m_szPtr, szString, Length) == 0)
	{
		if (this->m_szPtr[Length] == 0)
			return true;
	}

	return false;
}

void CString::LTrim()
{
	size_t NumToMove = 0;
	while (this->m_szPtr[NumToMove] == ' ' ||
		   this->m_szPtr[NumToMove] == '\t' ||
		   this->m_szPtr[NumToMove] == '\r' ||
		   this->m_szPtr[NumToMove] == '\n')
	{
		NumToMove++;
	}

	size_t Len = (this->Len() + 1) - NumToMove;
	memmove(this->m_szPtr, &this->m_szPtr[NumToMove], sizeof(char) * Len);
}

void CString::RTrim()
{
	size_t CurrentPos = this->Len();
	while (this->m_szPtr[CurrentPos] == ' ' ||
		   this->m_szPtr[CurrentPos] == '\t' ||
		   this->m_szPtr[CurrentPos] == '\r' ||
		   this->m_szPtr[CurrentPos] == '\n')
	{
		CurrentPos--;
	}

	this->m_szPtr[CurrentPos + 1] = 0;
}

size_t CString::Len()
{
	return strlen(this->m_szPtr);
}

const size_t CString::Len() const
{
	return strlen(this->m_szPtr);
}

const char* CString::GetCString() 
{
	return this->m_szPtr;
}

const char* CString::GetCString() const
{
	return this->m_szPtr;
}

void CString::_Allocate(const size_t& Length)
{
	size_t AllocSize = DEFAULT_ALLOC_SIZE;
	if (Length > DEFAULT_ALLOC_SIZE)
	{
		AllocSize = Length;
	}

	this->m_szPtr = new char[AllocSize];
	this->m_BufferLength = AllocSize;
}

void CString::_CheckGrow(const size_t& Length, const size_t& CurrentLength)
{
	// correct 0-char
	size_t CorrectLength = CurrentLength + 1;

	// check to grow
	if ((CorrectLength + Length) > this->m_BufferLength)
	{
		size_t SizeDiff = (CorrectLength + Length) - this->m_BufferLength;
		size_t GrowSize = DEFAULT_GROW_SIZE;

		if (SizeDiff > DEFAULT_GROW_SIZE)
		{
			GrowSize = SizeDiff;
		}

		size_t NewSize = this->m_BufferLength + GrowSize;
		char* szNew = new char[NewSize];
		memcpy(szNew, this->m_szPtr, sizeof(char) * CorrectLength);
		delete this->m_szPtr;
		this->m_szPtr = szNew;
		this->m_BufferLength = NewSize;
	}
}

};